@@ -215,7 +215,7 @@ bool KFileItemModelRolesUpdater::enlargeSmallPreviews() const
 
 void KFileItemModelRolesUpdater::setEnabledPlugins(const QStringList& list)
 {
-    if (m_enabledPlugins == list) {
+    if (m_enabledPlugins != list) {
         m_enabledPlugins = list;
         if (m_previewShown) {
             updateAllPreviews();
@@ -595,6 +595,13 @@ void KFileItemModelRolesUpdater::applyChangedNepomukRoles(const Nepomuk::Resourc
 #ifdef HAVE_NEPOMUK
     const KUrl itemUrl = m_nepomukUriItems.value(resource.resourceUri());
     const KFileItem item = m_model->fileItem(itemUrl);
+
+    if (item.isNull()) {
+        // itemUrl is not in the model anymore, probably because
+        // the corresponding file has been deleted in the meantime.
+        return;
+    }
+
     QHash<QByteArray, QVariant> data = rolesData(item);
 
     const KNepomukRolesProvider& rolesProvider = KNepomukRolesProvider::instance();
