@@ -25,22 +25,24 @@
 
 #include <libdolphin_export.h>
 
-#include <kitemviews/kitemlistgroupheader.h>
+#include <kitemviews/kstandarditemlistgroupheader.h>
 #include <kitemviews/kitemliststyleoption.h>
-#include <kitemviews/kitemlistviewanimation_p.h>
 #include <kitemviews/kitemlistwidget.h>
 #include <kitemviews/kitemmodelbase.h>
+#include <kitemviews/private/kitemlistviewanimation.h>
 #include <QGraphicsWidget>
 #include <QSet>
 
 class KItemListController;
 class KItemListGroupHeaderCreatorBase;
 class KItemListHeader;
+class KItemListHeaderWidget;
 class KItemListSizeHintResolver;
 class KItemListRubberBand;
 class KItemListViewAnimation;
 class KItemListViewLayouter;
 class KItemListWidget;
+class KItemListWidgetInformant;
 class KItemListWidgetCreatorBase;
 class KItemListViewCreatorBase;
 class QTimer;
@@ -52,11 +54,8 @@ class QTimer;
  * a GraphicsItem. Each visible item is represented by a KItemListWidget.
  *
  * The created view must be applied to the KItemListController with
- * KItemListController::setView(). For showing a custom model it is not
- * mandatory to derive from KItemListView, all that is necessary is
- * to set a widget-creator that is capable to create KItemListWidgets
- * showing the model items. A widget-creator can be set with
- * KItemListView::setWidgetCreator().
+ * KItemListController::setView() or with the constructor of
+ * KItemListController.
  *
  * @see KItemListWidget
  * @see KItemModelBase
@@ -72,17 +71,6 @@ public:
     KItemListView(QGraphicsWidget* parent = 0);
     virtual ~KItemListView();
 
-    /**
-     * If the scroll-orientation is vertical, the items are ordered
-     * from top to bottom (= default setting). If the scroll-orientation
-     * is horizontal, the items are ordered from left to right.
-     */
-    void setScrollOrientation(Qt::Orientation orientation);
-    Qt::Orientation scrollOrientation() const;
-
-    void setItemSize(const QSizeF& size);
-    QSizeF itemSize() const;
-
     /**
      * Offset of the scrollbar that represents the scroll-orientation
      * (see setScrollOrientation()).
@@ -144,17 +132,31 @@ public:
      * <code>
      * itemListView->setWidgetCreator(new KItemListWidgetCreator<X>());
      * </code>
-     * Note that the ownership of the widget creator is not transferred to
-     * the item-list view: One instance of a widget creator might get shared
-     * by several item-list view instances.
+     * The ownership of the widget creator is transferred to
+     * the item-list view.
      **/
     void setWidgetCreator(KItemListWidgetCreatorBase* widgetCreator);
     KItemListWidgetCreatorBase* widgetCreator() const;
 
+    /**
+     * Sets the creator that creates a group header. Usually it is sufficient
+     * to implement a custom header widget X derived from KItemListGroupHeader and
+     * set the creator by:
+     * <code>
+     * itemListView->setGroupHeaderCreator(new KItemListGroupHeaderCreator<X>());
+     * </code>
+     * The ownership of the gropup header creator is transferred to
+     * the item-list view.
+     **/
     void setGroupHeaderCreator(KItemListGroupHeaderCreatorBase* groupHeaderCreator);
     KItemListGroupHeaderCreatorBase* groupHeaderCreator() const;
 
-    void setStyleOption(const KItemListStyleOption& option);
+    /**
+     * @return The basic size of all items. The size of an item may be larger than
+     *         the basic size (see KItemListView::itemSizeHint() and KItemListView::itemRect()).
+     */
+    QSizeF itemSize() const;
+
     const KItemListStyleOption& styleOption() const;
 
     /** @reimp */
@@ -185,20 +187,11 @@ public:
 
     /**
      * @return Required size for the item with the index \p index.
-     *         Per default KItemListView::itemSize() is returned.
-     *         When reimplementing this method it is recommended to
-     *         also reimplement KItemListView::itemSizeHintUpdateRequired().
+     *         The returned value might be larger than KItemListView::itemSize().
+     *         In this case the layout grid will be stretched to assure an
+     *         unclipped item.
      */
-    virtual QSizeF itemSizeHint(int index) const;
-
-    /**
-     * @param itemRanges Items that must be checked for getting the visible roles sizes.
-     * @return           The size of each visible role in case if KItemListView::itemSize()
-     *                   is empty. This allows to have dynamic but equal role sizes between
-     *                   all items, like used in the classic "table-views". Per default an
-     *                   empty hash is returned.
-     */
-    virtual QHash<QByteArray, QSizeF> visibleRolesSizes(const KItemRangeList& itemRanges) const;
+    QSizeF itemSizeHint(int index) const;
 
     /**
      * If set to true, items having child-items can be expanded to show the child-items as
@@ -253,20 +246,30 @@ public:
     bool isTransactionActive() const;
 
     /**
-     * Turns on the header if \p show is true. Per default the
-     * header is not shown. Usually the header is turned on when
+     * Turns on the header if \p visible is true. Per default the
+     * header is not visible. Usually the header is turned on when
      * showing a classic "table-view" to describe the shown columns.
      */
-    void setHeaderShown(bool show);
-    bool isHeaderShown() const;
+    void setHeaderVisible(bool visible);
+    bool isHeaderVisible() const;
+
+    /**
+     * @return Header of the list. The header is also available if it is not shown
+     *         (see KItemListView::setHeaderShown()).
+     */
+    KItemListHeader* header() const;
 
     /**
      * @return Pixmap that is used for a drag operation based on the
-     *         items given by \a indexes. The default implementation returns
-     *         a null-pixmap.
+     *         items given by \a indexes.
      */
     virtual QPixmap createDragPixmap(const QSet<int>& indexes) const;
 
+    /**
+     * Lets the user edit the role \a role for item with the index \a index.
+     */
+    void editRole(int index, const QByteArray& role);
+
     /**
      * @reimp
      */
@@ -306,7 +309,41 @@ signals:
      */
     void visibleRolesChanged(const QList<QByteArray>& current, const QList<QByteArray>& previous);
 
+    void roleEditingCanceled(int index, const QByteArray& role, const QVariant& value);
+    void roleEditingFinished(int index, const QByteArray& role, const QVariant& value);
+
 protected:
+    void setItemSize(const QSizeF& size);
+    void setStyleOption(const KItemListStyleOption& option);
+
+    /**
+     * If the scroll-orientation is vertical, the items are ordered
+     * from top to bottom (= default setting). If the scroll-orientation
+     * is horizontal, the items are ordered from left to right.
+     */
+    void setScrollOrientation(Qt::Orientation orientation);
+    Qt::Orientation scrollOrientation() const;
+
+    /**
+     * Factory method for creating a default widget-creator. The method will be used
+     * in case if setWidgetCreator() has not been set by the application.
+     * @return New instance of the widget-creator that should be used per
+     *         default.
+     */
+    virtual KItemListWidgetCreatorBase* defaultWidgetCreator() const;
+
+    /**
+     * Factory method for creating a default group-header-creator. The method will be used
+     * in case if setGroupHeaderCreator() has not been set by the application.
+     * @return New instance of the group-header-creator that should be used per
+     *         default.
+     */
+    virtual KItemListGroupHeaderCreatorBase* defaultGroupHeaderCreator() const;
+
+    /**
+     * Is called when creating a new KItemListWidget instance and allows derived
+     * classes to do a custom initialization.
+     */
     virtual void initializeItemListWidget(KItemListWidget* item);
 
     /**
@@ -364,22 +401,22 @@ private slots:
     void slotRubberBandActivationChanged(bool active);
 
     /**
-     * Is invoked if the visible role-width of one role in the header has
-     * been changed by the user. It is remembered that the user has modified
-     * the role-width, so that it won't be changed anymore automatically to
-     * calculate an optimized width.
+     * Is invoked if the column-width of one role in the header has
+     * been changed by the user. The automatic resizing of columns
+     * will be turned off as soon as this method has been called at
+     * least once.
      */
-    void slotVisibleRoleWidthChanged(const QByteArray& role,
-                                     qreal currentWidth,
-                                     qreal previousWidth);
+    void slotHeaderColumnWidthChanged(const QByteArray& role,
+                                      qreal currentWidth,
+                                      qreal previousWidth);
 
     /**
-     * Is invoked if a visible role has been moved by the user. Applies
+     * Is invoked if a column has been moved by the user. Applies
      * the moved role to the view.
      */
-    void slotVisibleRoleMoved(const QByteArray& role,
-                              int currentIndex,
-                              int previousIndex);
+    void slotHeaderColumnMoved(const QByteArray& role,
+                               int currentIndex,
+                               int previousIndex);
 
     /**
      * Triggers the autoscrolling if autoScroll() is enabled by checking the
@@ -396,6 +433,9 @@ private slots:
      */
     void slotGeometryOfGroupHeaderParentChanged();
 
+    void slotRoleEditingCanceled(int index, const QByteArray& role, const QVariant& value);
+    void slotRoleEditingFinished(int index, const QByteArray& role, const QVariant& value);
+
 private:
     enum LayoutAnimationHint
     {
@@ -480,7 +520,7 @@ private:
     void updateGroupHeaderLayout(KItemListWidget* widget);
 
     /**
-     * Recycles the group-header from the widget.
+     * Recycles the group-header for the widget.
      */
     void recycleGroupHeaderForWidget(KItemListWidget* widget);
 
@@ -518,39 +558,47 @@ private:
     bool useAlternateBackgrounds() const;
 
     /**
-     * @return The widths of each visible role that is shown in the KItemListHeader.
+     * @param itemRanges Items that must be checked for getting the widths of columns.
+     * @return           The preferred width of the column of each visible role. The width will
+     *                   be respected if the width of the item size is <= 0 (see
+     *                   KItemListView::setItemSize()). Per default an empty hash
+     *                   is returned.
      */
-    QHash<QByteArray, qreal> headerRolesWidths() const;
+    QHash<QByteArray, qreal> preferredColumnWidths(const KItemRangeList& itemRanges) const;
 
     /**
-     * Updates m_visibleRolesSizes by calling KItemListView::visibleRolesSizes().
-     * Nothing will be done if m_itemRect is not empty or custom header-widths
-     * are used (see m_useHeaderWidths). Also m_strechedVisibleRolesSizes will be adjusted
-     * to respect the available view-size.
+     * Applies the column-widths from m_headerWidget to the layout
+     * of the view.
      */
-    void updateVisibleRolesSizes(const KItemRangeList& itemRanges);
+    void applyColumnWidthsFromHeader();
 
     /**
-     * Convenience method for updateVisibleRoleSizes(KItemRangeList() << KItemRange(0, m_model->count()).
+     * Applies the column-widths from m_headerWidget to \a widget.
      */
-    void updateVisibleRolesSizes();
+    void updateWidgetColumnWidths(KItemListWidget* widget);
 
     /**
-     * Updates m_stretchedVisibleRolesSizes based on m_visibleRolesSizes and the available
-     * view-size. Nothing will be done if m_itemRect is not empty or custom header-widths
-     * are used (see m_useHeaderWidths).
+     * Updates the preferred column-widths of m_groupHeaderWidget by
+     * invoking KItemListView::columnWidths().
      */
-    void updateStretchedVisibleRolesSizes();
+    void updatePreferredColumnWidths(const KItemRangeList& itemRanges);
 
     /**
-     * @return Sum of the widths of all visible roles.
+     * Convenience method for
+     * updatePreferredColumnWidths(KItemRangeList() << KItemRange(0, m_model->count()).
      */
-    qreal visibleRolesSizesWidthSum() const;
+    void updatePreferredColumnWidths();
 
     /**
-     * @return Sum of the heights of all visible roles.
+     * Resizes the column-widths of m_headerWidget based on the preferred widths
+     * and the vailable view-size.
      */
-    qreal visibleRolesSizesHeightSum() const;
+    void applyAutomaticColumnWidths();
+
+    /**
+     * @return Sum of the widths of all columns.
+     */
+    qreal columnWidthsSum() const;
 
     /**
      * @return Boundaries of the header. An empty rectangle is returned
@@ -582,6 +630,16 @@ private:
      */
     bool scrollBarRequired(const QSizeF& size) const;
 
+    /**
+     * Shows a drop-indicator between items dependent on the given
+     * cursor position. The cursor position is relative the the upper left
+     * edge of the view.
+     * @return Index of the item where the dropping is done. An index of -1
+     *         indicates that the item has been dropped after the last item.
+     */
+    int showDropIndicator(const QPointF& pos);
+    void hideDropIndicator();
+
     /**
      * Applies the height of the group header to the layouter. The height
      * depends on the used scroll orientation.
@@ -627,6 +685,7 @@ private:
     bool m_enabledSelectionToggles;
     bool m_grouped;
     bool m_supportsItemExpanding;
+    bool m_editingRole;
     int m_activeTransactions; // Counter for beginTransaction()/endTransaction()
     LayoutAnimationHint m_endTransactionAnimationHint;
 
@@ -634,10 +693,8 @@ private:
     KItemListController* m_controller;
     KItemModelBase* m_model;
     QList<QByteArray> m_visibleRoles;
-    QHash<QByteArray, QSizeF> m_visibleRolesSizes;
-    QHash<QByteArray, QSizeF> m_stretchedVisibleRolesSizes;
-    KItemListWidgetCreatorBase* m_widgetCreator;
-    KItemListGroupHeaderCreatorBase* m_groupHeaderCreator;
+    mutable KItemListWidgetCreatorBase* m_widgetCreator;
+    mutable KItemListGroupHeaderCreatorBase* m_groupHeaderCreator;
     KItemListStyleOption m_styleOption;
 
     QHash<int, KItemListWidget*> m_visibleItems;
@@ -671,9 +728,18 @@ private:
     QTimer* m_autoScrollTimer;
 
     KItemListHeader* m_header;
-    bool m_useHeaderWidths;
+    KItemListHeaderWidget* m_headerWidget;
+
+    // When dragging items into the view where the sort-role of the model
+    // is empty, a visual indicator should be shown during dragging where
+    // the dropping will happen. This indicator is specified by an index
+    // of the item. -1 means that no indicator will be shown at all.
+    // The m_dropIndicator is set by the KItemListController
+    // by KItemListView::showDropIndicator() and KItemListView::hideDropIndicator().
+    QRectF m_dropIndicator;
 
     friend class KItemListContainer; // Accesses scrollBarRequired()
+    friend class KItemListHeader;    // Accesses m_headerWidget
     friend class KItemListController;
     friend class KItemListControllerTest;
 };
@@ -703,29 +769,58 @@ private:
  * @brief Base class for creating KItemListWidgets.
  *
  * It is recommended that applications simply use the KItemListWidgetCreator-template class.
- * For a custom implementation the methods create() and recyle() must be reimplemented.
- * The intention of the widget creator is to prevent repetitive and expensive instantiations and
- * deletions of KItemListWidgets by recycling existing widget instances.
+ * For a custom implementation the methods create(), itemSizeHint() and preferredColumnWith()
+ * must be reimplemented. The intention of the widget creator is to prevent repetitive and
+ * expensive instantiations and deletions of KItemListWidgets by recycling existing widget
+ * instances.
  */
 class LIBDOLPHINPRIVATE_EXPORT KItemListWidgetCreatorBase : public KItemListCreatorBase
 {
 public:
     virtual ~KItemListWidgetCreatorBase();
+
     virtual KItemListWidget* create(KItemListView* view) = 0;
+
     virtual void recycle(KItemListWidget* widget);
+
+    virtual QSizeF itemSizeHint(int index, const KItemListView* view) const = 0;
+
+    virtual qreal preferredRoleColumnWidth(const QByteArray& role,
+                                           int index,
+                                           const KItemListView* view) const = 0;
 };
 
+/**
+ * @brief Template class for creating KItemListWidgets.
+ */
 template <class T>
 class KItemListWidgetCreator : public KItemListWidgetCreatorBase
 {
 public:
+    KItemListWidgetCreator();
     virtual ~KItemListWidgetCreator();
+
     virtual KItemListWidget* create(KItemListView* view);
+
+    virtual QSizeF itemSizeHint(int index, const KItemListView* view) const;
+
+    virtual qreal preferredRoleColumnWidth(const QByteArray& role,
+                                           int index,
+                                           const KItemListView* view) const;
+private:
+    KItemListWidgetInformant* m_informant;
 };
 
+template <class T>
+KItemListWidgetCreator<T>::KItemListWidgetCreator() :
+    m_informant(T::createInformant())
+{
+}
+
 template <class T>
 KItemListWidgetCreator<T>::~KItemListWidgetCreator()
 {
+    delete m_informant;
 }
 
 template <class T>
@@ -733,12 +828,26 @@ KItemListWidget* KItemListWidgetCreator<T>::create(KItemListView* view)
 {
     KItemListWidget* widget = static_cast<KItemListWidget*>(popRecycleableWidget());
     if (!widget) {
-        widget = new T(view);
+        widget = new T(m_informant, view);
         addCreatedWidget(widget);
     }
     return widget;
 }
 
+template<class T>
+QSizeF KItemListWidgetCreator<T>::itemSizeHint(int index, const KItemListView* view) const
+{
+    return m_informant->itemSizeHint(index, view);
+}
+
+template<class T>
+qreal KItemListWidgetCreator<T>::preferredRoleColumnWidth(const QByteArray& role,
+                                                          int index,
+                                                          const KItemListView* view) const
+{
+    return m_informant->preferredRoleColumnWidth(role, index, view);
+}
+
 /**
  * @brief Base class for creating KItemListGroupHeaders.
  *
