@@ -237,6 +237,14 @@ public:
     void setNameFilter(const QString& nameFilter);
     QString nameFilter() const;
 
+    /**
+     * Filters the currently shown items by \a filters. All items
+     * whose content-type matches those given by the list of filters
+     * will be shown.
+     */
+    void setMimeTypeFilters(const QStringList& filters);
+    QStringList mimeTypeFilters() const;
+
     /**
      * Returns a textual representation of the state of the current
      * folder or selected items, suitable for use in the status bar.
@@ -524,6 +532,7 @@ protected:
 
     /** @reimp */
     virtual void hideEvent(QHideEvent* event);
+    virtual bool event(QEvent* event);
 
 private slots:
     /**
@@ -756,6 +765,7 @@ private:
     // For unit tests
     friend class TestBase;
     friend class DolphinDetailsViewTest;
+    friend class DolphinPart;                   // Accesses m_model
 };
 
 /// Allow using DolphinView::Mode in QVariant
